/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file Adaptable.h
 *    Declaration of the IAdaptable interface.
 *
 * @author Erik Einhorn
 * @date   2010/12/29
 */

#ifndef _MIRA_ADAPTABLE_H_
#define _MIRA_ADAPTABLE_H_

#include <factory/Factory.h>

namespace mira {

//////////////////////////////////////////////////////////////////////////////

/**
 * An interface for an adaptable object.
 * Adaptable objects can be dynamically extended to provide different
 * interfaces (or "adapters"). Adapters are created by adapter factories.
 * (Docu taken from Eclipse Platform SDK)
 */
class IAdaptable
{
public:

	virtual ~IAdaptable() {}

	/**
	 * Returns an object which is an instance of the given class associated
	 * with this object. Returns nullptr if no such object can be found.
	 * The implementation if IAdaptable is responsible for destroying the
	 * instance of the created object.
	 */
	virtual Object* getAdapter(const Class& adapter) = 0;
};

//////////////////////////////////////////////////////////////////////////////

} // namespace

#endif
