/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file TapeChannelView.h
 *    View showing informations about a channel in a tape.
 *
 * @author Tim Langner
 * @date   2012/01/01
 */

#ifndef _MIRA_TAPECHANNELVIEW_H_
#define _MIRA_TAPECHANNELVIEW_H_

#include <QDialog>
#include <QComboBox>
#include <QDialogButtonBox>

#include <widgets/PropertyEditor.h>
#include <widgets/ScrollView.h>

#include <TapeChannelInfo.h>

class QHeaderView;

namespace mira {

///////////////////////////////////////////////////////////////////////////////

class TapeEditor;

class TapeChannelPropertyDialog : public QDialog
{
	Q_OBJECT

public:

	TapeChannelPropertyDialog(TapeChannelInfo* channel,
	                          TapeEditor* editor, QWidget* parent = NULL);

public slots:

	void changeRenderer(int index);

protected:

	TapeChannelInfo* mChannel;
	TapeEditor* mEditor;
	PropertyEditor* mPropertyEditor;
	boost::shared_ptr<PropertyNode> mPropertyNode;
	QDialogButtonBox* mButtonBox;
	QComboBox* mRenderer;
};

class TapeChannelView : public ScrollView
{
	Q_OBJECT;
public:

	TapeChannelView(QWidget* parent, TapeEditor* editor);

	QHeaderView* getHeader() { return mHeader; }

	void updateContentsHeight();
	virtual void updateContents(const QRect& rect);
	virtual void updateContents();

	TapeChannelInfo* getChannelAtPos(const QPoint& pos);
	TapeChannelInfo* getChannel(int idx);

protected slots:

	void onContentsMoving(int cx, int cy);
	void updateHeader();

protected:

	void resizeEvent(QResizeEvent* resizeEvent);
	void contextMenuEvent(QContextMenuEvent* contextMenuEvent);
	void mousePressEvent(QMouseEvent* mouseEvent);
	void mouseMoveEvent(QMouseEvent* mouseEvent);
	void mouseReleaseEvent(QMouseEvent* mouseEvent);
	void mouseDoubleClickEvent(QMouseEvent *mouseEvent);

	void updatePixmap(int cx, int cy);
	virtual void drawContents(QPainter* painter, const QRect& rect);

	void changeRendererDialog(TapeChannelInfo* info);
	void drawCell(QPainter* painter, const TapeChannelInfoMap::value_type& data, int col, const QRect& rect);

	TapeEditor* mEditor;
	QHeaderView* mHeader;
	QPixmap mPixmap;
	bool mInUpdate;

	std::vector<int> mRowSeparatorPositions; // updated in updateContentsHeight()
	int mResizeDragRow; // contains the row that is being resized, or -1 if nothing is resized
	int mResizeDragStartPos;
	int mResizeDragStartHeight;
};

///////////////////////////////////////////////////////////////////////////////

}

#endif 
